package stt

import (
	"fmt"
	"math"
	"strconv"
)

var pow10 []uint64

func init() {
	pow10 = []uint64{1, 10, 100, 1000, 10000, 100000, 1000000}
}

// WriteFloat32 write float32 to stream
func (stream *Stream) WriteFloat32(val float32) {
	if math.IsInf(float64(val), 0) || math.IsNaN(float64(val)) {
		stream.Error = fmt.Errorf("unsupported value: %f", val)
		return
	}
	abs := math.Abs(float64(val))
	ff := byte('f')
	// Note: Must use float32 comparisons for underlying float32 value to get precise cutoffs right.
	if abs != 0 {
		if float32(abs) < 1e-6 || float32(abs) >= 1e21 {
			ff = 'e'
		}
	}
	stream.buf = strconv.AppendFloat(stream.buf, float64(val), ff, -1, 32)
}

// WriteFloat64 write float64 to stream
func (stream *Stream) WriteFloat64(val float64) {
	if math.IsInf(val, 0) || math.IsNaN(val) {
		stream.Error = fmt.Errorf("unsupported value: %f", val)
		return
	}
	abs := math.Abs(val)
	ff := byte('f')
	// Note: Must use float32 comparisons for underlying float32 value to get precise cutoffs right.
	if abs != 0 {
		if abs < 1e-6 || abs >= 1e21 {
			ff = 'e'
		}
	}
	stream.buf = strconv.AppendFloat(stream.buf, val, ff, -1, 64)
}
